import subprocess

from pibot import leds

from time import sleep
from pibot.nano import Nano
from pibot.lcd import LCD
import RPi.GPIO as GPIO


def startup():
    lcd = LCD(fontsize=16)
    GPIO.setwarnings(False)

    # show TU-Chemnitz logo on screen
    lcd.view_image("/home/pi/.resources/tu.pgm", 1)

    sleep(2)
    # show TU-Chemnitz text on screen
    lcd.view_image("/home/pi/.resources/tutext.pgm", 1)

    sleep(2)
    # show Roboschool logo on screen
    lcd.view_image("/home/pi/.resources/roboschool.pgm", 1)

    sleep(2)
    # show PiBot logo on screen
    lcd.view_image("/home/pi/.resources/pibot.pgm", 1)

    # getting IP
    cmd = "hostname -I | cut -d\' \' -f1"
    ip = (subprocess.check_output(cmd, shell=True)).decode("utf-8")

    # getting voltage
    nano = Nano()
    voltage = nano.get_battery_voltage()

    message = [ "IP: ",
                str(ip), # on newline for IP's with mostly 3 digits (xxx.xxx.xxx.xxx)
                "Battery: {} mV".format(voltage)]
    lcd.print(message)
    sleep(2)



import RPi.GPIO as GPIO

from pibot import constants as c
from time import sleep


def init_leds():
    GPIO.setmode(GPIO.BCM)
    GPIO.setwarnings(False)

    pins = [c.LED_LEFT_RED_PIN,
            c.LED_LEFT_GREEN_PIN,
            c.LED_MID_RED_PIN,
            c.LED_MID_GREEN_PIN,
            c.LED_RIGHT_RED_PIN,
            c.LED_RIGHT_GREEN_PIN,
            c.LED_FRONT_LEFT_PIN,
            c.LED_FRONT_RIGHT_PIN
            ]

    GPIO.setup(pins, direction=GPIO.OUT, initial=0)


def set_led(led, color):
    if led == c.LED_LEFT:
        if color == c.RED:
            GPIO.output(c.LED_LEFT_RED_PIN, GPIO.HIGH)
            GPIO.output(c.LED_LEFT_GREEN_PIN, GPIO.LOW)
        elif color == c.GREEN:
            GPIO.output(c.LED_LEFT_RED_PIN, GPIO.LOW)
            GPIO.output(c.LED_LEFT_GREEN_PIN, GPIO.HIGH)
        elif color == c.YELLOW:
            GPIO.output(c.LED_LEFT_RED_PIN, GPIO.HIGH)
            GPIO.output(c.LED_LEFT_GREEN_PIN, GPIO.HIGH)
        elif color == c.OFF:
            GPIO.output(c.LED_LEFT_RED_PIN, GPIO.LOW)
            GPIO.output(c.LED_LEFT_GREEN_PIN, GPIO.LOW)
        else:
            raise Exception("Color {} not supported".format(color))
    elif led == c.LED_MID:
        if color == c.RED:
            GPIO.output(c.LED_MID_RED_PIN, GPIO.HIGH)
            GPIO.output(c.LED_MID_GREEN_PIN, GPIO.LOW)
        elif color == c.GREEN:
            GPIO.output(c.LED_MID_RED_PIN, GPIO.LOW)
            GPIO.output(c.LED_MID_GREEN_PIN, GPIO.HIGH)
        elif color == c.YELLOW:
            GPIO.output(c.LED_MID_RED_PIN, GPIO.HIGH)
            GPIO.output(c.LED_MID_GREEN_PIN, GPIO.HIGH)
        elif color == c.OFF:
            GPIO.output(c.LED_MID_RED_PIN, GPIO.LOW)
            GPIO.output(c.LED_MID_GREEN_PIN, GPIO.LOW)
        else:
            raise Exception("Color {} not supported".format(color))
    elif led == c.LED_RIGHT:
        if color == c.RED:
            GPIO.output(c.LED_RIGHT_RED_PIN, GPIO.HIGH)
            GPIO.output(c.LED_RIGHT_GREEN_PIN, GPIO.LOW)
        elif color == c.GREEN:
            GPIO.output(c.LED_RIGHT_RED_PIN, GPIO.LOW)
            GPIO.output(c.LED_RIGHT_GREEN_PIN, GPIO.HIGH)
        elif color == c.YELLOW:
            GPIO.output(c.LED_RIGHT_RED_PIN, GPIO.HIGH)
            GPIO.output(c.LED_RIGHT_GREEN_PIN, GPIO.HIGH)
        elif color == c.OFF:
            GPIO.output(c.LED_RIGHT_RED_PIN, GPIO.LOW)
            GPIO.output(c.LED_RIGHT_GREEN_PIN, GPIO.LOW)
        else:
            raise Exception("Color {} not supported".format(color))
    elif led == c.LED_FRONT_LEFT:
        if color == c.ON:
            GPIO.output(c.LED_FRONT_LEFT_PIN, GPIO.HIGH)
        elif color == c.OFF:
            GPIO.output(c.LED_FRONT_LEFT_PIN, GPIO.LOW)
        else:
            raise Exception("Color {} not supported".format(color))
    elif led == c.LED_FRONT_RIGHT:
        if color == c.ON:
            GPIO.output(c.LED_FRONT_RIGHT_PIN, GPIO.HIGH)
        elif color == c.OFF:
            GPIO.output(c.LED_FRONT_RIGHT_PIN, GPIO.LOW)
        else:
            raise Exception("Color {} not supported".format(color))


def check_leds():
    # front
    # ON
    GPIO.output(c.LED_FRONT_LEFT_PIN, GPIO.HIGH)
    GPIO.output(c.LED_FRONT_RIGHT_PIN, GPIO.HIGH)
    sleep(0.1)
    GPIO.output(c.LED_FRONT_LEFT_PIN, GPIO.LOW)
    GPIO.output(c.LED_FRONT_RIGHT_PIN, GPIO.LOW)
    sleep(0.1)
    GPIO.output(c.LED_FRONT_LEFT_PIN, GPIO.HIGH)
    GPIO.output(c.LED_FRONT_RIGHT_PIN, GPIO.HIGH)
    sleep(0.1)
    GPIO.output(c.LED_FRONT_LEFT_PIN, GPIO.LOW)
    GPIO.output(c.LED_FRONT_RIGHT_PIN, GPIO.LOW)

    # others
    # RED LEFT
    GPIO.output(c.LED_LEFT_RED_PIN, GPIO.HIGH)
    GPIO.output(c.LED_LEFT_GREEN_PIN, GPIO.LOW)
    sleep(0.05)
    # RED MID
    GPIO.output(c.LED_MID_RED_PIN, GPIO.HIGH)
    GPIO.output(c.LED_MID_GREEN_PIN, GPIO.LOW)
    sleep(0.05)
    # RED RIGHT
    GPIO.output(c.LED_RIGHT_RED_PIN, GPIO.HIGH)
    GPIO.output(c.LED_RIGHT_GREEN_PIN, GPIO.LOW)
    sleep(0.5)

    # YELLOW LEFT
    GPIO.output(c.LED_LEFT_RED_PIN, GPIO.HIGH)
    GPIO.output(c.LED_LEFT_GREEN_PIN, GPIO.HIGH)
    sleep(0.05)
    # YELLOW  MID
    GPIO.output(c.LED_MID_RED_PIN, GPIO.HIGH)
    GPIO.output(c.LED_MID_GREEN_PIN, GPIO.HIGH)
    sleep(0.05)
    # YELLOW RIGHT
    GPIO.output(c.LED_RIGHT_RED_PIN, GPIO.HIGH)
    GPIO.output(c.LED_RIGHT_GREEN_PIN, GPIO.HIGH)
    sleep(0.5)

    # GREEN LEFT
    GPIO.output(c.LED_LEFT_RED_PIN, GPIO.LOW)
    GPIO.output(c.LED_LEFT_GREEN_PIN, GPIO.HIGH)
    sleep(0.05)
    # GREEN MID
    GPIO.output(c.LED_MID_RED_PIN, GPIO.LOW)
    GPIO.output(c.LED_MID_GREEN_PIN, GPIO.HIGH)
    sleep(0.05)
    # GREEN RIGHT
    GPIO.output(c.LED_RIGHT_RED_PIN, GPIO.LOW)
    GPIO.output(c.LED_RIGHT_GREEN_PIN, GPIO.HIGH)
    sleep(1)

    # OFF LEFT
    GPIO.output(c.LED_LEFT_RED_PIN, GPIO.LOW)
    GPIO.output(c.LED_LEFT_GREEN_PIN, GPIO.LOW)
    # OFF MID
    GPIO.output(c.LED_MID_RED_PIN, GPIO.LOW)
    GPIO.output(c.LED_MID_GREEN_PIN, GPIO.LOW)
    # OFF RIGHT
    GPIO.output(c.LED_RIGHT_RED_PIN, GPIO.LOW)
    GPIO.output(c.LED_RIGHT_GREEN_PIN, GPIO.LOW)



import RPi.GPIO as GPIO
import subprocess
from os import popen
from time import sleep
from picamera import PiCamera
from PIL import Image
from io import BytesIO

from pibot.lcd import LCD
from pibot.nano import Nano
from pibot.buttons import *
from pibot import constants as c
from pibot import leds


MENU_TOGGLE_PIN = 21

def background():
    GPIO.setmode(GPIO.BCM)
    GPIO.setup(MENU_TOGGLE_PIN, GPIO.IN, GPIO.PUD_UP)
    lcd = LCD(fontsize=21)

    while 1:
        if is_pressed(MENU_TOGGLE_PIN):
            menu(lcd)
        sleep(0.5)


def menu(lcd):
    init_buttons()

    menu_entries = [('Exit', exit_menu),
                    ('Poweroff', poweroff),
                    ('Show IP', show_ip),
                    ('Motor Test', actuator_test),
                    ('Sensor Test', sensor_test),
                    ('LED Test', led_test),
                    ('Camera Test', camera_test),
                    ('Buzzer Test', buzzer_test)  ]

    current = 1
    in_menu = True
    while in_menu:
        message = [ str(menu_entries[current][0]),
                    "",
                    "<-       ok       ->"]
        lcd.print(message)
        button = wait_for_any()
        if button == c.BUTTON_LEFT:
            current -= 1
        elif button == c.BUTTON_MID:
            menu_entries[current][1](lcd)
            if current == 0 or current == 1:
                in_menu = False
                lcd.clear()
        elif button == c.BUTTON_RIGHT:
            current += 1
        current %= len(menu_entries)
        sleep(0.02)


def exit_menu(lcd):
    pass


def poweroff(lcd):
    message = [ "Are you sure?",
                "",
                "Yes            No"]
    lcd.print(message)
    button = wait_for_any()
    if button == c.BUTTON_LEFT:
        message = [ "  Shutdown...",
                    "",
                    ""]
        lcd.print(message)
        sleep(1)
        popen("sudo poweroff")


def show_ip(lcd):
    cmd = "hostname -I | cut -d\' \' -f1"
    ip = (subprocess.check_output(cmd, shell=True)).decode("utf-8")
    leds.init_leds()
    leds.set_led(c.LED_MID, c.GREEN)
    message = [ "IP:",
                str(ip),
                "              "]
    lcd.set_fontsize(17)
    lcd.print(message)
    lcd.set_fontsize(21)
    wait_for_button(c.BUTTON_MID)
    leds.set_led(c.LED_MID, c.OFF)


def actuator_test(lcd):
    message = [ "Requesting",
                "Nano!",
                ""]
    lcd.print(message)
    nano = Nano()
    while nano.get_battery_voltage() == 0:
        message = [ "Please plug in",
                    "the 9V battery!",
                    "              "]
        lcd.print(message)
        sleep(0.5)

    message = [ "Make sure the",
                "motors are",
                "powered on!"]
    lcd.print(message)
    leds.init_leds()
    leds.set_led(c.LED_MID, c.GREEN)
    wait_for_button(c.BUTTON_MID)
    leds.set_led(c.LED_MID, c.OFF)

    nano.reset_encoders()
    nano.set_motors(-30, 30)
    sleep(2)
    nano.set_motors(30, -30)
    sleep(2)
    nano.set_motors(0, 0)
    message = [str(nano.get_encoders())]
    lcd.print(message)

    leds.set_led(c.LED_MID, c.GREEN)
    wait_for_button(c.BUTTON_MID)
    leds.set_led(c.LED_MID, c.OFF)


def sensor_test(lcd):
    message = [ "Requesting",
                "Nano!",
                ""]
    lcd.print(message)
    nano = Nano()
    leds.init_leds()
    leds.set_led(c.LED_MID, c.GREEN)
    while not is_pressed(c.BUTTON_MID):
        message = [ str(nano.get_distances()),
                    "",
                    "          ok"]
        lcd.print(message)
    leds.set_led(c.LED_MID, c.OFF)


def led_test(lcd):
    message = [ "Testing",
                "LED's!",
                ""]
    lcd.print(message)
    leds.init_leds()
    leds.check_leds()


IMAGE_PATH = "/home/pi/Pictures/"

def camera_test(lcd):
    leds.init_leds()
    with PiCamera() as cam:
        cam.resolution = lcd.get_resolution()
        cam.rotation = 180

        leds.set_led(c.LED_MID, c.GREEN)
        leds.set_led(c.LED_FRONT_LEFT, c.ON)
        leds.set_led(c.LED_FRONT_RIGHT, c.ON)
        while not is_pressed(c.BUTTON_MID):
            stream = BytesIO()
            cam.capture(stream, 'jpeg')
            stream.seek(0)
            image = Image.open(stream)
            lcd.view_pil_image(image)
        leds.set_led(c.LED_MID, c.OFF)
        leds.set_led(c.LED_FRONT_LEFT, c.OFF)
        leds.set_led(c.LED_FRONT_RIGHT, c.OFF)
        wait_for_button_release(c.BUTTON_MID)


def buzzer_test(lcd):
    message = [ "Requesting",
                "Nano!",
                ""]
    lcd.print(message)
    nano = Nano()
    lcd.clear()
    nano.set_buzzer(660, 100)
    sleep(0.150)
    nano.set_buzzer(660, 100)
    sleep(0.300)
    nano.set_buzzer(660, 100)
    sleep(0.300)
    nano.set_buzzer(510, 100)
    sleep(0.100)
    nano.set_buzzer(660, 100)
    sleep(0.300)
    nano.set_buzzer(770, 100)
    sleep(0.550)
    nano.set_buzzer(380, 100)
    sleep(0.575)




from time import sleep

import cv2
import imutils as imutils
from picamera.array import PiRGBArray
from picamera import PiCamera


class Camera:
    def __init__(self, width=640, height=480):
        """
        Initializes the camera
        :param width: width of the images captured by the camera
        :param height: height of the images captured by the camera
        """
        self.width = width
        self.height = height
        self.RED_LOWER = (125, 0, 0)
        self.RED_UPPER = (255, 100, 100)
        self.BLUE_LOWER = (0, 0, 125)
        self.BLUE_UPPER = (100, 100, 255)
        self.windows = []

    def rotate_image(self, orig, angle):
        """
        Rotates an image
        :param orig: The image to be rotated
        :param angle: The angle of the rotation
        :return: The rotated image
        """
        dimensions = (self.width // 2, self.height // 2)
        mat = cv2.getRotationMatrix2D(dimensions, angle, 1)
        rotated = cv2.warpAffine(orig, mat, (self.width, self.height))
        return rotated

    def capture_image(self):
        """
        Captures one image from the camera
        :return: the image captured by the camera
        """
        with PiCamera() as camera:
            camera.resolution = (self.width, self.height)
            with PiRGBArray(camera, size=(self.width, self.height)) as output:
                output.truncate(0)
                camera.capture(output, 'bgr', use_video_port=True)
                return self.rotate_image(output.array.copy(), 180)

    def capture_multiple_images(self, amount, delay):
        """
        Captures multiple images
        :param amount: (int) How many images should be captured
        :param delay: (int or float) The time between two captures
        :return: (list) a list of the captured images
        """
        with PiCamera() as camera:
            camera.resolution = (self.width, self.height)
            with PiRGBArray(camera, size=(self.width, self.height)) as output:
                images = []
                if amount > 128:
                    amount = 128
                for _ in range(amount):
                    output.truncate(0)
                    camera.capture(output, 'bgr', use_video_port=True)
                    images.append(self.rotate_image(output.array.copy(), 180))
                    sleep(delay)
                return images

    def check_for_ball(self, image, color, radius):
        """
        Checks if the image contains a ball with the specified color
        :param image: (2d color array) The image to be checked
        :param color: (str) Either red or blue
        :param radius: (int or float) The radius which the ball needs to be recognized as a ball (in px)
        :return: (tuple) returns two values: boolean whether the ball is detected or not and the (manipulated) image
        """
        if image is not None:
            rgb = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)
            if color == "red":
                mask = cv2.inRange(rgb, self.RED_LOWER, self.RED_UPPER)
                cnts = cv2.findContours(mask.copy(), cv2.RETR_EXTERNAL,
                                        cv2.CHAIN_APPROX_SIMPLE)
                cnts = imutils.grab_contours(cnts)
            elif color == "blue":
                mask = cv2.inRange(rgb, self.BLUE_LOWER, self.BLUE_UPPER)
                cnts = cv2.findContours(mask.copy(), cv2.RETR_EXTERNAL,
                                        cv2.CHAIN_APPROX_SIMPLE)
                cnts = imutils.grab_contours(cnts)
            else:
                raise Exception("Only check for red or blue ball is supported. Given: {}".format(color))

            if len(cnts) > 0:
                # find the largest contour in the mask, then use
                # it to compute the minimum enclosing circle and
                # centroid
                c = max(cnts, key=cv2.contourArea)
                ((x, y), r) = cv2.minEnclosingCircle(c)
                M = cv2.moments(c)
                if M["m00"] > 0:
                    center = (int(M["m10"] / M["m00"]), int(M["m01"] / M["m00"]))

                    # only proceed if the radius meets a minimum size
                    if r > radius:
                        # draw the circle and centroid on the frame,
                        # then update the list of tracked points
                        image_manipulated = image.copy()
                        cv2.circle(image_manipulated, (int(x), int(y)), int(r), (0, 255, 255), 2)
                        cv2.circle(image_manipulated, center, 5, (0, 0, 255), -1)
                        return True, image_manipulated
            return False, image
        else:
            raise Exception("You haven't captured an image yet")

    def view_image(self, image, window_name):
        """
        Views an image
        :param image: The image to be viewed
        :param window_name: The name of the window where the image should be put in
        """
        cv2.imshow(window_name, image)
        if window_name not in self.windows:
            cv2.waitKey(1000)



from __future__ import print_function, division, unicode_literals, absolute_import

import struct

from enum import Enum


class Order(Enum):
    """
    Pre-defined orders
    """

    START = 0
    CONNECTED = 1
    RECEIVED = 2
    ERROR = 3
    MOTORS = 4
    VOLTAGE = 5
    DISTANCES = 6
    GET_ENCODERS = 7
    RESET_ENCODERS = 8
    STAT_LED = 9
    BUZZER = 10


def read_order(f):
    """
    :param f: file handler or serial file
    :return: (Order Enum Object)
    """
    return Order(read_i8(f))


def read_i8(f):
    """
    :param f: file handler or serial file
    :return: (int8_t)
    """
    return struct.unpack('<b', bytearray(f.read(1)))[0]


def read_ui8(f):
    """
    :param f: file handler or serial file
    :return: (uint8_t)
    """
    return struct.unpack('<B', bytearray(f.read(1)))[0]


def read_i16(f):
    """
    :param f: file handler or serial file
    :return: (int16_t)
    """
    return struct.unpack('<h', bytearray(f.read(2)))[0]


def read_i32(f):
    """
    :param f: file handler or serial file
    :return: (int32_t)
    """
    return struct.unpack('<l', bytearray(f.read(4)))[0]


def write_i8(f, value):
    """
    :param f: file handler or serial file
    :param value: (int8_t)
    """
    if -128 <= value <= 127:
        f.write(struct.pack('<b', value))
    else:
        print("Value error:{}".format(value))


def write_order(f, order):
    """
    :param f: file handler or serial file
    :param order: (Order Enum Object)
    """
    write_i8(f, order.value)


def write_i16(f, value):
    """
    :param f: file handler or serial file
    :param value: (int16_t)
    """
    f.write(struct.pack('<h', value))


def write_ui16(f, value):
    """
    :param f: file handler or serial file
    :param value: (uint16_t)
    """
    f.write(struct.pack('<H', value))


def write_i32(f, value):
    """
    :param f: file handler or serial file
    :param value: (int32_t)
    """
    f.write(struct.pack('<l', value))



import Adafruit_SSD1306

from PIL import Image
from PIL import ImageDraw
from PIL import ImageFont
from PIL import ImageOps


class LCD:
    def __init__(self, fonttype="/home/pi/.fonts/Roboto-Regular.ttf", fontsize=14):
        self.disp = Adafruit_SSD1306.SSD1306_128_64(rst=None)

        # Initialize library.
        self.disp.begin()

        self.fonttype = fonttype
        self.font = ImageFont.truetype(fonttype, fontsize)

    def print(self, text):
        image = Image.new('1', (self.disp.width, self.disp.height))
        draw = ImageDraw.Draw(image)
        for i in range(len(text)):
            draw.text((0, -2 + i * (self.font.size + 2)), text[i], font=self.font, fill=255)
        self.disp.image(image)
        self.disp.display()

    def view_image(self, source, invert):
        image = Image.open(source).resize((self.disp.width, self.disp.height), Image.ANTIALIAS)
        if invert:
            image = image.convert('L')
            image = ImageOps.invert(image)
        self.view_pil_image(image)

    def view_pil_image(self, image):
        image = image.convert('1')
        self.disp.image(image)
        self.disp.display()

    def set_fontsize(self, fontsize):
        self.font = ImageFont.truetype(self.fonttype, fontsize)

    def clear(self):
        self.disp.clear()
        self.disp.display()

    def get_resolution(self):
        return self.disp.width, self.disp.height



# "Enums"
LED_LEFT = 42
LED_MID = 43
LED_RIGHT = 44
LED_FRONT_LEFT = 45
LED_FRONT_RIGHT = 46
RED = 47
GREEN = 48
YELLOW = 49
ON = 50
OFF = 51
BUTTON_LEFT = 52
BUTTON_MID = 53
BUTTON_RIGHT = 54

# Pin definitions
LED_LEFT_RED_PIN = 19
LED_LEFT_GREEN_PIN = 0

LED_MID_RED_PIN = 5
LED_MID_GREEN_PIN = 6

LED_RIGHT_RED_PIN = 16
LED_RIGHT_GREEN_PIN = 13

LED_FRONT_RIGHT_PIN = 25
LED_FRONT_LEFT_PIN = 26

BUTTON_LEFT_PIN = 1
BUTTON_MID_PIN = 12
BUTTON_RIGHT_PIN = 20



import RPi.GPIO as GPIO
from pibot import constants as c

from time import sleep


def init_buttons():
    GPIO.setmode(GPIO.BCM)
    GPIO.setwarnings(False)

    GPIO.setup(c.BUTTON_LEFT_PIN, GPIO.IN, GPIO.PUD_UP)
    GPIO.setup(c.BUTTON_MID_PIN, GPIO.IN, GPIO.PUD_UP)
    GPIO.setup(c.BUTTON_RIGHT_PIN, GPIO.IN, GPIO.PUD_UP)


def wait_for_button_press(button):
    while not is_pressed(button):
        sleep(0.001)


def wait_for_button_release(button):
    while is_pressed(button):
        sleep(0.001)


def wait_for_button(button):
    wait_for_button_press(button)
    wait_for_button_release(button)

def wait_for_any():
    while True:
        if is_pressed(c.BUTTON_LEFT):
            return c.BUTTON_LEFT
        elif is_pressed(c.BUTTON_MID_PIN):
            return c.BUTTON_MID
        elif is_pressed(c.BUTTON_RIGHT_PIN):
            return c.BUTTON_RIGHT
        sleep(0.001)


def is_pressed(button):
    if button == c.BUTTON_LEFT:
        pin = c.BUTTON_LEFT_PIN
    elif button == c.BUTTON_MID:
        pin = c.BUTTON_MID_PIN
    elif button == c.BUTTON_RIGHT:
        pin = c.BUTTON_RIGHT_PIN
    else:
        pin = button

    if pin is not None:
        if not GPIO.input(pin):
            return True
    return False



from __future__ import print_function, division, absolute_import

import sys
import glob

try:
    import queue
except ImportError:
    import Queue as queue

from serial import Serial, SerialException


# From https://stackoverflow.com/questions/12090503/listing-available-com-ports-with-python
def get_serial_ports():
    """
    Lists serial ports.
    :return: ([str]) A list of available serial ports
    """
    if sys.platform.startswith('win'):
        ports = ['COM%s' % (i + 1) for i in range(256)]
    elif sys.platform.startswith('linux') or sys.platform.startswith('cygwin'):
        # this excludes your current terminal "/dev/tty"
        ports = glob.glob('/dev/tty[A-Za-z]*')
    elif sys.platform.startswith('darwin'):
        ports = glob.glob('/dev/tty.*')
    else:
        raise EnvironmentError('Unsupported platform')

    results = []
    for port in ports:
        try:
            s = Serial(port)
            s.close()
            results.append(port)
        except (OSError, SerialException):
            pass
    return results


def open_serial_port(serial_port=None, baudrate=115200, timeout=None, write_timeout=0):
    """
    Try to open serial port with Arduino
    If not port is specified, it will be automatically detected
    :param serial_port: (str)
    :param baudrate: (int)
    :param timeout: (int) None -> blocking mode
    :param write_timeout: (int)
    :return: (Serial Object)
    """
    # Open serial port (for communication with Arduino)
    if serial_port is None:
        serial_port = get_serial_ports()[0]
    # timeout=0 non-blocking mode, return immediately in any case, returning zero or more,
    # up to the requested number of bytes
    return Serial(port=serial_port, baudrate=baudrate, timeout=timeout, writeTimeout=write_timeout)


def clamp(value, minimun, maximum):
    return min(maximum, max(minimun, value))



import time

import RPi.GPIO as GPIO
from time import sleep
from pibot.serial_comm import write_order, Order, read_i16, write_i8, read_ui8, write_ui16
from pibot.utils import open_serial_port, clamp
# from OS import popen

import struct


class Nano:
    def __init__(self):
        # Reset Nano
        print("Resetting Nano...")
        self.reset_nano()

        print("Waiting for Serial Connection to Nano...")
        # sleep(5)

        try:
            self.serial_file = open_serial_port(baudrate=115200, timeout=None)
        except Exception as e:
            raise e

        is_connected = False

        # Initialize communication with Arduino
        print("Requesting Connection to Nano...")
        while not is_connected:
            write_order(self.serial_file, Order.START)
            if self.serial_file.inWaiting():
                byte = self.serial_file.read(1)
                if struct.unpack('B', byte)[0] == Order.CONNECTED.value:
                    is_connected = True
            # else:
            #     time.sleep(1)
        print("Connected to Nano.")

        # voltage check (minimum: 6500 mV, for testing: 7000mV)

        voltage = self.get_battery_voltage()
        if voltage < 7000:
            print("Warning! Battery voltage low! ({} mV)".format(voltage))
            # popen("sudo poweroff")
        else:
            print("Battery voltage okay! ({} mV)".format(voltage))

    def set_stat_led(self, toggle):
        write_order(self.serial_file, order=Order.STAT_LED)
        write_i8(self.serial_file, toggle)

    def set_motors(self, left, right):
        write_order(self.serial_file, order=Order.MOTORS)
        # Write left
        write_i8(self.serial_file, clamp(left, -128, 127))
        # Write right
        write_i8(self.serial_file, clamp(right, -128, 127))

    def get_encoders(self):
        write_order(self.serial_file, order=Order.GET_ENCODERS)

        # Wait for 2 values
        left = read_i16(self.serial_file)
        right = read_i16(self.serial_file)
        return left, right

    def reset_encoders(self):
        write_order(self.serial_file, order=Order.RESET_ENCODERS)

    def get_battery_voltage(self):
        write_order(self.serial_file, order=Order.VOLTAGE)
        return read_i16(self.serial_file)

    def get_distances(self):
        write_order(self.serial_file, order=Order.DISTANCES)

        # Wait for 3 values
        left = read_ui8(self.serial_file)
        mid = read_ui8(self.serial_file)
        right = read_ui8(self.serial_file)
        return left, mid, right

    def set_buzzer(self, freq, duration):
        write_order(self.serial_file, order=Order.BUZZER)
        # send the frequency
        write_ui16(self.serial_file, freq)
        # send the duration
        write_ui16(self.serial_file, duration)

    @staticmethod
    def reset_nano():
        GPIO.setmode(GPIO.BCM)
        GPIO.setwarnings(False)
        GPIO.setup(4, GPIO.OUT)
        GPIO.output(4, GPIO.LOW)
        sleep(0.001)
        GPIO.output(4, GPIO.HIGH)



import setuptools
import subprocess

with open("README.md", "r") as fh:
    long_description = fh.read()

version = subprocess.run(['git', 'rev-list', '--all', '--count'], stdout=subprocess.PIPE)
version = version.stdout.decode('utf-8').replace('\n', '').replace('\r', '')

req = []
with open('requirements.txt', 'r') as file:
    for line in file:
        if line != '' or line != '\n':
            req.append(line)

setuptools.setup(
    name="pibot",
    version=version,
    author="TUC-RoboSchool",
    author_email="basti.neubert@gmail.com",
    description="Library for controlling the PiBot",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/tuc-roboschool/PiBot",
    packages=setuptools.find_packages(),
    package_dir={'pibot': 'pibot'},
    install_requires=req,
    classifiers=[
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)",
        "Operating System :: OS Independent",
    ],
)
